
import java.io.FileReader;
import java.util.*;

class TestSortAlgorithm {
    private static int NR_SORTS = 100;

    public static void main(String[] args) {
        try {
            readAndTestSorting(args);
        } catch (Exception e) {
            e.printStackTrace();
            System.err.println();
            System.err.println("Usage: java TestSortAlgorithm <class-to-test> <textfile> <nr-words> <nr-tests>");
            System.err.println("  where <class-to-test> is: InsertionSort, MergeSort, HeapSort, QuickSort");
            System.err.println();
            System.exit(1);
        }
    }
    
    public static void readAndTestSorting(String[] args) throws Exception {
        if (args.length != 4) 
            throw new IllegalArgumentException();
        String classToTest = args[0];
        String textfile = args[1];
        int nrWords = Integer.parseInt(args[2]);
        int nrTests = Integer.parseInt(args[3]);

        SortAlgorithm algorithm = createAlgorithm(classToTest);
        String[] wordlist = new String[nrWords];
        String[] testlist = new String[nrWords];

        System.out.format("Reading %d words from textfile: %s\n\n", nrWords, textfile);
        Scanner scanner = new Scanner(new FileReader(textfile));
        for (int i = 0; i < wordlist.length; i++) {
            if (!scanner.hasNext()) throw new IndexOutOfBoundsException("Too few words in file!");
            wordlist[i] = scanner.next();
        }
        scanner.close();

        System.out.format("            Time/sort        Nr. words        Nr. sorts\n");
        for (int i = 0; i < nrTests; i++) {
            long testingTime = 0;
            for (int j = 0; j < NR_SORTS; j++) {
                System.arraycopy(wordlist, 0, testlist, 0, wordlist.length);
                testingTime -= System.nanoTime();
                algorithm.sort(testlist);
                testingTime += System.nanoTime();
            }
            System.out.format("%13.3f ms/sort %10d words %10d sorts\n", 
                              testingTime/1e6/NR_SORTS, nrWords, NR_SORTS);
        }
        System.out.println();
    }

    private static SortAlgorithm createAlgorithm(String className) throws IllegalArgumentException {
        if (className.equals("InsertionSort")) return new InsertionSort();
        if (className.equals("HeapSort"))      return new HeapSort();
        if (className.equals("QuickSort"))     return new QuickSort();
        if (className.equals("MergeSort"))     return new MergeSort();
        throw new IllegalArgumentException();
    }

}

